/******************************************************************************
 * Copyright (c) 2013 Oracle
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Konstantin Komissarchik - initial implementation and ongoing maintenance
 ******************************************************************************/

package org.eclipse.jst.config.web;

import org.eclipse.jst.config.web.internal.FilterReferenceService;
import org.eclipse.jst.config.web.internal.ServletReferenceService;
import org.eclipse.sapphire.Element;
import org.eclipse.sapphire.ElementList;
import org.eclipse.sapphire.ElementType;
import org.eclipse.sapphire.ListProperty;
import org.eclipse.sapphire.ReferenceValue;
import org.eclipse.sapphire.Since;
import org.eclipse.sapphire.Validation;
import org.eclipse.sapphire.Validations;
import org.eclipse.sapphire.Value;
import org.eclipse.sapphire.ValueProperty;
import org.eclipse.sapphire.modeling.annotations.CountConstraint;
import org.eclipse.sapphire.modeling.annotations.Documentation;
import org.eclipse.sapphire.modeling.annotations.Fact;
import org.eclipse.sapphire.modeling.annotations.Facts;
import org.eclipse.sapphire.modeling.annotations.Image;
import org.eclipse.sapphire.modeling.annotations.Label;
import org.eclipse.sapphire.modeling.annotations.MustExist;
import org.eclipse.sapphire.modeling.annotations.NoDuplicates;
import org.eclipse.sapphire.modeling.annotations.PossibleValues;
import org.eclipse.sapphire.modeling.annotations.Reference;
import org.eclipse.sapphire.modeling.annotations.Required;
import org.eclipse.sapphire.modeling.annotations.Service;
import org.eclipse.sapphire.modeling.annotations.Type;
import org.eclipse.sapphire.modeling.xml.annotations.XmlBinding;

/**
 * @author <a href="mailto:konstantin.komissarchik@oracle.com">Konstantin Komissarchik</a>
 */

@Label( standard = "filter mapping" )

@Documentation
(
    content = "Associates a filter with one or more target. A target can be either a servlet or a URL pattern."
)

public interface FilterMapping extends Element
{
    ElementType TYPE = new ElementType( FilterMapping.class );
    
    // *** Filter ***
    
    @Reference( target = Filter.class )
    @Service( impl = FilterReferenceService.class )
    @Label( standard = "filter" )
    @Required
    @MustExist
    @PossibleValues( property = "/Filters/Name" )
    @XmlBinding( path = "filter-name" )
    
    @Documentation
    (
        content = "The name of a defined filter."
    )
    
    ValueProperty PROP_FILTER = new ValueProperty( TYPE, "Filter" );
    
    ReferenceValue<String,Filter> getFilter();
    void setFilter( String value );
    
    // *** Targets ***

    @Label( standard = "target" )
    
    interface Target extends Element
    {
        ElementType TYPE = new ElementType( Target.class );
        
        @Required
        @XmlBinding( path = "" )

        ValueProperty PROP_SPECIFICATION = new ValueProperty( TYPE, "Specification" );
        
        Value<String> getSpecification();
        void setSpecification( String value );
    }
    
    @Label( standard = "servlet" )
    @Image( path = "Servlet.png" )
    @XmlBinding( path = "servlet-name" )
    
    interface ServletTarget extends Target
    {
        ElementType TYPE = new ElementType( ServletTarget.class );
        
        @Reference( target = Servlet.class )
        @Service( impl = ServletReferenceService.class )
        @Label( standard = "servlet" )
        @MustExist
        @PossibleValues( property = "/Servlets/Name" )
        
        ValueProperty PROP_SPECIFICATION = new ValueProperty( TYPE, Target.PROP_SPECIFICATION );
    }
    
    @Label( standard = "URL pattern" )
    @Image( path = "UrlPattern.png" )
    @XmlBinding( path = "url-pattern" )
    
    interface UrlPatternTarget extends Target
    {
        ElementType TYPE = new ElementType( UrlPatternTarget.class );
        
        @Label( standard = "URL pattern" )
        
        ValueProperty PROP_SPECIFICATION = new ValueProperty( TYPE, Target.PROP_SPECIFICATION );
    }
    
    @Type( base = Target.class, possible = { ServletTarget.class, UrlPatternTarget.class } )
    @Label( standard = "targets" )
    
    @Validations
    (
        {
            @Validation
            (
                rule = "${ Targets.Size > 0 }",
                message = "At least one target must be specified."
            ),
            @Validation
            (
                rule = "${ Targets.Size <= 1 || ( Targets.Size > 1 && Parent.ServletPlatformVersion.VersionMatches( '[2.5' ) ) }",
                message = "Multiple targets are supported by Servlet Platform 2.5 or newer."
            )
        }
    )
    
    @Facts
    (
        {
            @Fact( statement = "Must have at least one." ),
            @Fact( statement = "Multiple targets are supported by Servlet Platform 2.5 or newer." )
        }
    )
    
    ListProperty PROP_TARGETS = new ListProperty( TYPE, "Targets" );
    
    ElementList<Target> getTargets();
    
    // *** DispatcherEvents ***
    
    @Label( standard = "dispatcher event" )
    @XmlBinding( path = "dispatcher" )

    public interface DispatcherEventsListEntry extends Element
    {
        ElementType TYPE = new ElementType( DispatcherEventsListEntry.class );
        
        @Type( base = DispatcherEvent.class )
        @Label( standard = "dispatcher event" )
        @Required
        @NoDuplicates
        @XmlBinding( path = "" )
        
        ValueProperty PROP_DISPATCHER_EVENT = new ValueProperty( TYPE, "DispatcherEvent" );
        
        Value<DispatcherEvent> getDispatcherEvent();
        void setDispatcherEvent( String value );
        void setDispatcherEvent( DispatcherEvent value );
    }
    
    @Type( base = DispatcherEventsListEntry.class )
    @Label( standard = "dispatcher events" )
    @CountConstraint( max = 4 )
    @Since( "2.4" )
    
    @Documentation
    (
        content = "Control which dispatcher events the filter should map to. There are four dispatcher events: " +
                  "forward, request, include and error.[br/][br/]" +
                  
                  "If forward event is referenced, the filter will be applied under RequestDispatcher.forward() " +
                  "calls.[br/][br/]" +
                  
                  "If request event is referenced, the filter will be applied under ordinary client calls to the " +
                  "path or a servlet.[br/][br/]" +
                  
                  "If include event is referenced, the filter will be applied under RequestDispatcher.include() " +
                  "calls.[br/][br/]" +
                  
                  "If error event is referenced, the filter will be applied under the error page mechanism.[br/][br/]" +
                  
                  "The absence of any dispatcher event references indicates a default of applying the filter only " +
                  "under ordinary client calls to the path or a servlet."
    )
    
    ListProperty PROP_DISPATCHER_EVENTS = new ListProperty( TYPE, "DispatcherEvents" );
    
    ElementList<DispatcherEventsListEntry> getDispatcherEvents();
    
}
