/******************************************************************************
 * Copyright (c) 2013 Oracle
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Konstantin Komissarchik - initial implementation and ongoing maintenance
 ******************************************************************************/

package org.eclipse.jst.config.web;

import org.eclipse.jst.config.DescribableExt;
import org.eclipse.jst.config.internal.InvertingBooleanXmlValueBinding;
import org.eclipse.sapphire.Since;
import org.eclipse.sapphire.ListProperty;
import org.eclipse.sapphire.ElementList;
import org.eclipse.sapphire.ElementType;
import org.eclipse.sapphire.Value;
import org.eclipse.sapphire.ValueProperty;
import org.eclipse.sapphire.modeling.annotations.CountConstraint;
import org.eclipse.sapphire.modeling.annotations.DefaultValue;
import org.eclipse.sapphire.modeling.annotations.Documentation;
import org.eclipse.sapphire.modeling.annotations.Label;
import org.eclipse.sapphire.modeling.annotations.Type;
import org.eclipse.sapphire.modeling.xml.annotations.CustomXmlValueBinding;
import org.eclipse.sapphire.modeling.xml.annotations.XmlBinding;
import org.eclipse.sapphire.modeling.xml.annotations.XmlListBinding;

/**
 * @author <a href="mailto:konstantin.komissarchik@oracle.com">Konstantin Komissarchik</a>
 */

@Label( standard = "JSP property group" )

public interface JspPropertyGroup extends DescribableExt
{
    ElementType TYPE = new ElementType( JspPropertyGroup.class );
    
    // *** UrlPatterns ***
    
    @Type( base = UrlPattern.class )
    @Label( standard = "URL patterns" )
    @CountConstraint( min = 1 )
    @XmlListBinding( mappings = @XmlListBinding.Mapping( element = "url-pattern", type = UrlPattern.class ) )
    
    @Documentation
    (
        content = "The set of URL patterns define the group of JSP pages that these properties should " +
                  "affect. A URL pattern must follow the rules specified in Section 11.2 of the Servlet API " +
                  "Specification. This pattern is assumed to be in URL-decoded form and must not contain " +
                  "CR (#xD) or LF (#xA). The container must preserve all other characters including " +
                  "whitespace."
    )
    
    ListProperty PROP_URL_PATTERNS = new ListProperty( TYPE, "UrlPatterns" );
    
    ElementList<UrlPattern> getUrlPatterns();
    
    // *** PageEncoding ***
    
    @Label( standard = "page encoding" )
    @XmlBinding( path = "page-encoding" )

    @Documentation
    (
        content = "The valid values are those of pageEncoding directive. It is a translation-time " +
                  "error to name different encodings in the pageEncoding attribute of the page directive " +
                  "of a JSP page and in a JSP configuration element matching the page. It is also a " +
                  "translation-time error to name different encodings in the prolog or text declaration " +
                  "of a document in XML syntax and in a JSP configuration element matching the document. " +
                  "It is legal to name the same encoding through multiple mechanisms."
    )
    
    // TODO: Research default value and possible values.
    
    ValueProperty PROP_PAGE_ENCODING = new ValueProperty( TYPE, "PageEncoding" );
    
    Value<String> getPageEncoding();
    void setPageEncoding( String value );
    
    // *** ExpressionsLanguageEnabled ***
    
    @Type( base = Boolean.class )
    @Label( standard = "expression language enabled" )
    @DefaultValue( text = "true" )
    @CustomXmlValueBinding( impl = InvertingBooleanXmlValueBinding.class, params = "el-ignored" )

    @Documentation
    (
        content = "Specifies whether expressions language (EL) evaluation is enabled for the matched group of JSP pages. " +
                  "By default, the EL evaluation is enabled for web applications using Servlet 2.4 or greater spec, " +
                  "and disabled otherwise."
    )
    
    // TODO: Default varies based on spec level
    
    ValueProperty PROP_EXPRESSIONS_LANGUAGE_ENABLED = new ValueProperty( TYPE, "ExpressionsLanguageEnabled" );
    
    Value<Boolean> getExpressionsLanguageEnabled();
    void setExpressionsLanguageEnabled( String value );
    void setExpressionsLanguageEnabled( Boolean value );
    
    // *** ScriptingEnabled ***
    
    @Type( base = Boolean.class )
    @Label( standard = "scripting enabled" )
    @DefaultValue( text = "true" )
    @CustomXmlValueBinding( impl = InvertingBooleanXmlValueBinding.class, params = "scripting-invalid" )
    
    @Documentation
    (
        content = "Specifies whether scripting is enabled for the matched group of JSP pages."
    )
    
    ValueProperty PROP_SCRIPTING_ENABLED = new ValueProperty( TYPE, "ScriptingEnabled" );
    
    Value<Boolean> getScriptingEnabled();
    void setScriptingEnabled( String value );
    void setScriptingEnabled( Boolean value );
    
    // *** XmlContent ***
    
    @Type( base = Boolean.class )
    @Label( standard = "XML content" )
    @DefaultValue( text = "false" )
    @XmlBinding( path = "is-xml" )
    
    @Documentation
    (
        content = "If true, denotes that the matched group of resources are JSP documents, and thus must be " +
                  "interpreted as XML documents." +
                  "[pbr/]" +
                  "If false, the resources are assumed to not be JSP documents, unless there is another property " +
                  "group that indicates otherwise."
    )
    
    // TODO: Determine the actual default value
    
    ValueProperty PROP_XML_CONTENT = new ValueProperty( TYPE, "XmlContent" );
    
    Value<Boolean> getXmlContent();
    void setXmlContent( String value );
    void setXmlContent( Boolean value );
    
    // *** Preludes ***
    
    @Type( base = JspInclude.class )
    @Label( standard = "preludes" )
    @XmlListBinding( mappings = @XmlListBinding.Mapping( element = "include-prelude", type = JspInclude.class ) )
    
    @Documentation
    (
        content = "Preludes are treated as an automatic include (similar to the include directive) at the beginning of " +
                  "each matched JSP page. Each prelude must specify a path (relative to the root of the web application) " +
                  "where to find the resource to be included."
    )
    
    ListProperty PROP_PRELUDES = new ListProperty( TYPE, "Preludes" );
    
    ElementList<JspInclude> getPreludes();
    
    // *** Codas ***
    
    @Type( base = JspInclude.class )
    @Label( standard = "codas" )
    @XmlListBinding( mappings = @XmlListBinding.Mapping( element = "include-coda", type = JspInclude.class ) )
    
    @Documentation
    (
        content = "Codas are treated as an automatic include (similar to the include directive) at the end of " +
                  "each matched JSP page. Each coda must specify a path (relative to the root of the web application) " +
                  "where to find the resource to be included."
    )
    
    ListProperty PROP_CODAS = new ListProperty( TYPE, "Codas" );
    
    ElementList<JspInclude> getCodas();
    
    // *** DeferredSyntaxAllowedAsLiteral ***
    
    @Type( base = Boolean.class )
    @Label( standard = "deferred syntax allowed as literal" )
    @DefaultValue( text = "false" )
    @Since( "2.5" )
    @XmlBinding( path = "deferred-syntax-allowed-as-literal" )
    
    @Documentation
    (
        content = "The character sequence #{ is reserved for EL expressions. Consequently, a translation error " + 
                  "occurs if the #{ character sequence is used as a String literal, unless this option is enabled."
    )
    
    ValueProperty PROP_DEFERRED_SYNTAX_ALLOWED_AS_LITERAL = new ValueProperty( TYPE, "DeferredSyntaxAllowedAsLiteral" );
    
    Value<Boolean> getDeferredSyntaxAllowedAsLiteral();
    void setDeferredSyntaxAllowedAsLiteral( String value );
    void setDeferredSyntaxAllowedAsLiteral( Boolean value );
    
    // *** TrimDirectiveWhitespaces ***
    
    @Type( base = Boolean.class )
    @Label( standard = "trim directive whitespaces" )
    @DefaultValue( text = "false" )
    @Since( "2.5" )
    @XmlBinding( path = "trim-directive-whitespaces" )
    
    @Documentation
    (
        content = "Indicates that template text containing only whitespaces must be removed from the response " + 
                  "output. It has no effect on JSP documents (XML syntax)."
    )
    
    ValueProperty PROP_TRIM_DIRECTIVE_WHITESPACES = new ValueProperty( TYPE, "TrimDirectiveWhitespaces" );
    
    Value<Boolean> getTrimDirectiveWhitespaces();
    void setTrimDirectiveWhitespaces( String value );
    void setTrimDirectiveWhitespaces( Boolean value );
    
}
