/******************************************************************************
 * Copyright (c) 2013 Oracle
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Konstantin Komissarchik - initial implementation and ongoing maintenance
 ******************************************************************************/

package org.eclipse.jst.config.web;

import org.eclipse.jst.config.Describable;
import org.eclipse.jst.config.web.internal.SecurityConstraintServices;
import org.eclipse.sapphire.Since;
import org.eclipse.sapphire.ElementProperty;
import org.eclipse.sapphire.Element;
import org.eclipse.sapphire.ListProperty;
import org.eclipse.sapphire.ElementHandle;
import org.eclipse.sapphire.ElementList;
import org.eclipse.sapphire.ElementType;
import org.eclipse.sapphire.Value;
import org.eclipse.sapphire.ValueProperty;
import org.eclipse.sapphire.modeling.annotations.CountConstraint;
import org.eclipse.sapphire.modeling.annotations.DefaultValue;
import org.eclipse.sapphire.modeling.annotations.DependsOn;
import org.eclipse.sapphire.modeling.annotations.Documentation;
import org.eclipse.sapphire.modeling.annotations.Label;
import org.eclipse.sapphire.modeling.annotations.PossibleValues;
import org.eclipse.sapphire.modeling.annotations.Required;
import org.eclipse.sapphire.modeling.annotations.Type;
import org.eclipse.sapphire.modeling.xml.annotations.CustomXmlElementBinding;
import org.eclipse.sapphire.modeling.xml.annotations.CustomXmlListBinding;
import org.eclipse.sapphire.modeling.xml.annotations.XmlBinding;
import org.eclipse.sapphire.modeling.xml.annotations.XmlListBinding;

/**
 * @author <a href="mailto:konstantin.komissarchik@oracle.com">Konstantin Komissarchik</a>
 */

@Label( standard = "security constraint" )

@Documentation
(
    content = "Associates a security constraint with a collection of web resources."
)

public interface SecurityConstraint extends Element
{
    ElementType TYPE = new ElementType( SecurityConstraint.class );
    
    // *** Name ***
    
    @Label( standard = "name" )
    @Since( "2.3" )
    @XmlBinding( path = "display-name" )
    
    ValueProperty PROP_NAME = new ValueProperty( TYPE, "Name" );
    
    Value<String> getName();
    void setName( String value );
    
    // *** WebResourceCollections ***
    
    @Type( base = WebResourceCollection.class )
    @Label( standard = "web resource collections" )
    @CountConstraint( min = 1 )
    @XmlListBinding( mappings = @XmlListBinding.Mapping( element = "web-resource-collection", type = WebResourceCollection.class ) )
    
    ListProperty PROP_WEB_RESOURCE_COLLECTIONS = new ListProperty( TYPE, "WebResourceCollections" );
    
    ElementList<WebResourceCollection> getWebResourceCollections();
    
    // *** TransportGuarantee ***
    
    enum TransportGuarantee
    {
        NONE,
        INTEGRAL,
        CONFIDENTIAL
    }
    
    @Type( base = TransportGuarantee.class )
    @Label( standard = "transport guarantee" )
    @DefaultValue( text = "none" )
    @XmlBinding( path = "user-data-constraint/transport-guarantee" )
    
    @Documentation
    (
        content = "Specifies expectation regarding communication between the client and the server." +
                  "[pbr/]" +
                  "[b]None[/b] - The data does not require any transport guarantees." +
                  "[pbr/]" +
                  "[b]Integral[/b] - The data must be sent in such a way that it cannot be changed in transit." +
                  "[pbr/]" +
                  "[b]Confidential[/b] - The data must be sent in the manner that prevents other entities " +
                  "from observing the contents of the transmission." +
                  "[pbr/]" +
                  "In most cases, specifying integral or confidential transport guarantee will " +
                  "indicate that the use of SSL is required."
    )
    
    ValueProperty PROP_TRANSPORT_GUARANTEE = new ValueProperty( TYPE, "TransportGuarantee" );
    
    Value<TransportGuarantee> getTransportGuarantee();
    void setTransportGuarantee( String value );
    void setTransportGuarantee( TransportGuarantee value );
    
    // *** AccessControl ***
    
    @Label( standard = "access control" )
    
    public interface AccessControl extends Describable
    {
        ElementType TYPE = new ElementType( AccessControl.class );
    }
    
    @Label( standard = "closed" )
    
    public interface ClosedAccess extends AccessControl
    {
        ElementType TYPE = new ElementType( ClosedAccess.class );
    }
    
    @Label( standard = "open" )
    
    public interface OpenAccess extends AccessControl
    {
        ElementType TYPE = new ElementType( OpenAccess.class );
    }
    
    @Label( standard = "restricted" )
    
    public interface RestrictedAccess extends AccessControl
    {
        ElementType TYPE = new ElementType( RestrictedAccess.class );
        
        // *** Roles ***
        
        @Label( standard = "security role reference" )

        public interface RoleRef extends Element
        {
            ElementType TYPE = new ElementType( RoleRef.class );

            // *** Role ***
            
            @Label( standard = "role" )
            @Required
            @PossibleValues( property = "/SecurityRoles/Name" )
            @XmlBinding( path = "" )
            
            ValueProperty PROP_ROLE = new ValueProperty( TYPE, "Role" );
            
            Value<String> getRole();
            void setRole( String value );
        }
        
        @Type( base = RoleRef.class )
        @Label( standard = "roles" )
        @CountConstraint( min = 1 )
        @CustomXmlListBinding( impl = SecurityConstraintServices.RestrictedAccessRolesBinding.class )
        
        ListProperty PROP_ROLES = new ListProperty( TYPE, "Roles" );
        
        ElementList<RoleRef> getRoles();
    }
    
    @Type( base = AccessControl.class, possible = { ClosedAccess.class, OpenAccess.class, RestrictedAccess.class } )
    @Label( standard = "access control" )
    @CustomXmlElementBinding( impl = SecurityConstraintServices.AccessControlBinding.class )
    @DependsOn( "AccessControl[#type=RestrictedAccess]/Roles/Role" )
    
    @Documentation
    (
        content = "Access to web resource collections can be restricted based on roles. The specific roles " +
                  "that are allowed access can be listed explicitly, any user can be allowed access or access " +
                  "can be forbidden to all users."
    )
    
    ElementProperty PROP_ACCESS_CONTROL = new ElementProperty( TYPE, "AccessControl" );
    
    ElementHandle<AccessControl> getAccessControl();

}
