/******************************************************************************
 * Copyright (c) 2013 Oracle
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Konstantin Komissarchik - initial implementation and ongoing maintenance
 ******************************************************************************/

package org.eclipse.jst.config.web;

import org.eclipse.jst.config.web.internal.ServletReferenceService;
import org.eclipse.sapphire.Element;
import org.eclipse.sapphire.ElementList;
import org.eclipse.sapphire.ElementType;
import org.eclipse.sapphire.ListProperty;
import org.eclipse.sapphire.ReferenceValue;
import org.eclipse.sapphire.Validation;
import org.eclipse.sapphire.Validations;
import org.eclipse.sapphire.Value;
import org.eclipse.sapphire.ValueProperty;
import org.eclipse.sapphire.modeling.annotations.Documentation;
import org.eclipse.sapphire.modeling.annotations.Fact;
import org.eclipse.sapphire.modeling.annotations.Facts;
import org.eclipse.sapphire.modeling.annotations.Image;
import org.eclipse.sapphire.modeling.annotations.Label;
import org.eclipse.sapphire.modeling.annotations.MustExist;
import org.eclipse.sapphire.modeling.annotations.PossibleValues;
import org.eclipse.sapphire.modeling.annotations.Reference;
import org.eclipse.sapphire.modeling.annotations.Required;
import org.eclipse.sapphire.modeling.annotations.Service;
import org.eclipse.sapphire.modeling.annotations.Type;
import org.eclipse.sapphire.modeling.xml.annotations.XmlBinding;
import org.eclipse.sapphire.modeling.xml.annotations.XmlListBinding;

/**
 * @author <a href="mailto:konstantin.komissarchik@oracle.com">Konstantin Komissarchik</a>
 */

@Label( standard = "servlet mapping" )

@Documentation
(
    content = "Associates a servlet with one or more URL pattern."
)

public interface ServletMapping extends Element
{
    ElementType TYPE = new ElementType( ServletMapping.class );
    
    // *** Servlet ***
    
    @Reference( target = Servlet.class )
    @Service( impl = ServletReferenceService.class )
    @Label( standard = "servlet" )
    @Required
    @MustExist
    @PossibleValues( property = "/Servlets/Name" )
    @XmlBinding( path = "servlet-name" )
    
    @Documentation
    (
        content = "The name of a defined servlet."
    )
    
    ValueProperty PROP_SERVLET = new ValueProperty( TYPE, "Servlet" );
    
    ReferenceValue<String,Servlet> getServlet();
    void setServlet( String value );
    
    // *** UrlPatterns ***
    
    @Label( standard = "URL pattern" )
    @Image( path = "UrlPattern.png" )
    @XmlBinding( path = "url-pattern" )
    
    interface UrlPatternsListEntry extends Element
    {
        ElementType TYPE = new ElementType( UrlPatternsListEntry.class );
        
        @Label( standard = "URL pattern" )
        @Required
        @XmlBinding( path = "" )
        
        ValueProperty PROP_URL_PATTERN = new ValueProperty( TYPE, "UrlPattern" );
        
        Value<String> getUrlPattern();
        void setUrlPattern( String value );
    }
    
    @Type( base = UrlPatternsListEntry.class )
    @Label( standard = "URL patterns" )
    @XmlListBinding( path = "" )
    
    @Validations
    (
        {
            @Validation
            (
                rule = "${ UrlPatterns.Size > 0 }",
                message = "At least one URL pattern must be specified."
            ),
            @Validation
            (
                rule = "${ UrlPatterns.Size <= 1 || ( UrlPatterns.Size > 1 && Parent.ServletPlatformVersion.VersionMatches( '[2.5' ) ) }",
                message = "Multiple URL patterns are supported by Servlet Platform 2.5 or newer."
            )
        }
    )
    
    @Facts
    (
        {
            @Fact( statement = "Must have at least one." ),
            @Fact( statement = "Multiple URL patterns are supported by Servlet Platform 2.5 or newer." )
        }
    )
    
    ListProperty PROP_URL_PATTERNS = new ListProperty( TYPE, "UrlPatterns" );
    
    ElementList<UrlPatternsListEntry> getUrlPatterns();
    
}
