/******************************************************************************
 * Copyright (c) 2013 Oracle
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Konstantin Komissarchik - initial implementation and ongoing maintenance
 ******************************************************************************/

package org.eclipse.jst.config.web;

import org.eclipse.jst.config.web.internal.WebResourceCollectionServices;
import org.eclipse.sapphire.Element;
import org.eclipse.sapphire.ListProperty;
import org.eclipse.sapphire.ElementList;
import org.eclipse.sapphire.ElementType;
import org.eclipse.sapphire.Value;
import org.eclipse.sapphire.ValueProperty;
import org.eclipse.sapphire.modeling.annotations.CountConstraint;
import org.eclipse.sapphire.modeling.annotations.Documentation;
import org.eclipse.sapphire.modeling.annotations.Fact;
import org.eclipse.sapphire.modeling.annotations.Label;
import org.eclipse.sapphire.modeling.annotations.LongString;
import org.eclipse.sapphire.modeling.annotations.NoDuplicates;
import org.eclipse.sapphire.modeling.annotations.Required;
import org.eclipse.sapphire.modeling.annotations.Service;
import org.eclipse.sapphire.modeling.annotations.Type;
import org.eclipse.sapphire.modeling.xml.annotations.XmlBinding;
import org.eclipse.sapphire.modeling.xml.annotations.XmlListBinding;

/**
 * @author <a href="mailto:konstantin.komissarchik@oracle.com">Konstantin Komissarchik</a>
 */

@Label( standard = "web resource collection" )

@Documentation
(
    content = "Identifies a subset of web application resources and HTTP methods on those resources " +
              "to which a security constraint applies. If no HTTP methods are specified, then the " +
              "security constraint applies to all HTTP methods."
)

public interface WebResourceCollection extends Element
{
    ElementType TYPE = new ElementType( WebResourceCollection.class );
    
    // *** Name ***
    
    @Label( standard = "name" )
    @Required
    @XmlBinding( path = "web-resource-name" )
    
    ValueProperty PROP_NAME = new ValueProperty( TYPE, "Name" );
    
    Value<String> getName();
    void setName( String value );
    
    // *** Description ***
    
    @Label( standard = "description" )
    @LongString
    @XmlBinding( path = "description" )
    
    ValueProperty PROP_DESCRIPTION = new ValueProperty( TYPE, "Description" );
    
    Value<String> getDescription();
    void setDescription( String value );
    
    // *** UrlPatterns ***
    
    @Type( base = UrlPattern.class )
    @Label( standard = "URL patterns" )
    @CountConstraint( min = 1 )
    @XmlListBinding( mappings = @XmlListBinding.Mapping( element = "url-pattern", type = UrlPattern.class ) )
    
    ListProperty PROP_URL_PATTERNS = new ListProperty( TYPE, "UrlPatterns" );
    
    ElementList<UrlPattern> getUrlPatterns();
    
    // *** HttpMethods ***
    
    @Label( standard = "HTTP method" )

    interface HttpMethod extends Element
    {
        ElementType TYPE = new ElementType( HttpMethod.class );
        
        @Label( standard = "HTTP method" )
        @Required
        @NoDuplicates
        @XmlBinding( path = "" )
        
        ValueProperty PROP_NAME = new ValueProperty( TYPE, "Name" );
        
        Value<String> getName();
        void setName( String value );
    }

    @Type( base = HttpMethod.class )
    @Label( standard = "HTTP methods" )
    @Service( impl = WebResourceCollectionServices.HttpMethodPossibleValuesService.class )
    @Fact( statement = "Prior to Servlet Platform 2.5, restricted to seven standard HTTP/1.1 methods." )
    @XmlListBinding( mappings = @XmlListBinding.Mapping( element = "http-method", type = HttpMethod.class ) )

    @Documentation
    (
        content = "A security constraint can apply to a subset of HTTP methods. If no HTTP methods are specified, then the " +
                  "security constraint applies to all HTTP methods."
    )

    ListProperty PROP_HTTP_METHODS = new ListProperty( TYPE, "HttpMethods" );
    
    ElementList<HttpMethod> getHttpMethods();

}
