/******************************************************************************
 * Copyright (c) 2013 Oracle
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Konstantin Komissarchik - initial implementation and ongoing maintenance
 ******************************************************************************/

package org.eclipse.jst.config.web.internal;

import static org.eclipse.sapphire.modeling.xml.XmlUtil.createQualifiedName;

import java.util.List;

import javax.xml.namespace.QName;

import org.eclipse.jst.config.web.SecurityConstraint;
import org.eclipse.sapphire.ElementType;
import org.eclipse.sapphire.Resource;
import org.eclipse.sapphire.modeling.LayeredElementBindingImpl;
import org.eclipse.sapphire.modeling.xml.ChildXmlResource;
import org.eclipse.sapphire.modeling.xml.StandardXmlListBindingImpl;
import org.eclipse.sapphire.modeling.xml.XmlElement;
import org.eclipse.sapphire.modeling.xml.XmlNamespaceResolver;
import org.eclipse.sapphire.modeling.xml.XmlResource;

/**
 * @author <a href="mailto:konstantin.komissarchik@oracle.com">Konstantin Komissarchik</a>
 */

public final class SecurityConstraintServices
{
    private static final String ANY_ROLE_WILDCARD = "*";

    private static final String EL_AUTH_CONSTRAINT = "auth-constraint";
    private static final String EL_ROLE_NAME = "role-name";
    private static final String MC_RESTRICTED = "restricted";
    
    public static final class AccessControlBinding extends LayeredElementBindingImpl
    {
        @Override
        protected Object readUnderlyingObject()
        {
            final XmlElement elSecurityConstraint = ( (XmlResource) property().element().resource() ).getXmlElement();
            return elSecurityConstraint.getChildElement( EL_AUTH_CONSTRAINT, false );
        }

        @Override
        protected Resource createResource( final Object obj )
        {
            final XmlElement elAuthConstraint = (XmlElement) obj;
            final XmlResource parentXmlResource = (XmlResource) property().element().resource();
            
            return new ChildXmlResource( parentXmlResource, elAuthConstraint );
        }

        @Override
        public ElementType type( final Resource resource )
        {
            final XmlElement elAuthConstraint = ( (XmlResource) resource ).getXmlElement();
            final List<XmlElement> elRoleNameList = elAuthConstraint.getChildElements( EL_ROLE_NAME );
            final int elRoleNameCount = elRoleNameList.size();
            final boolean hasMetaComment = ( elAuthConstraint.getMetaComment( MC_RESTRICTED, false ) != null );
            
            if( elRoleNameCount > 0 )
            {
                if( elRoleNameCount == 1 && elRoleNameList.get( 0 ).getText().equals( ANY_ROLE_WILDCARD ) && ! hasMetaComment )
                {
                    return SecurityConstraint.OpenAccess.TYPE;
                }
                else
                {
                    return SecurityConstraint.RestrictedAccess.TYPE;
                }
            }
            else if( hasMetaComment )
            {
                return SecurityConstraint.RestrictedAccess.TYPE;
            }
            
            return SecurityConstraint.ClosedAccess.TYPE;
        }

        @Override
        protected Object createUnderlyingObject( final ElementType type )
        {
            final XmlElement elSecurityConstraint = ( (XmlResource) property().element().resource() ).getXmlElement();
            final XmlElement elAuthConstraint = elSecurityConstraint.getChildElement( EL_AUTH_CONSTRAINT, true );
            final List<XmlElement> elRoleNameList = elAuthConstraint.getChildElements( EL_ROLE_NAME );
            final int elRoleNameCount = elRoleNameList.size();
            final boolean hasMetaComment = ( elAuthConstraint.getMetaComment( MC_RESTRICTED, false ) != null );
            
            if( type == SecurityConstraint.ClosedAccess.TYPE )
            {
                elAuthConstraint.setMetaCommentText( MC_RESTRICTED, null );
                
                for( int i = 0; i < elRoleNameCount; i++ )
                {
                    elRoleNameList.get( i ).remove();
                }
                
            }
            else if( type == SecurityConstraint.OpenAccess.TYPE )
            {
                elAuthConstraint.setMetaCommentText( MC_RESTRICTED, null );
                
                for( int i = 1; i < elRoleNameCount; i++ )
                {
                    elRoleNameList.get( i ).remove();
                }
                
                elAuthConstraint.setChildNodeText( EL_ROLE_NAME, ANY_ROLE_WILDCARD, false );
            }
            else if( type == SecurityConstraint.RestrictedAccess.TYPE )
            {
                if( elRoleNameCount == 0 )
                {
                    elAuthConstraint.setMetaCommentText( MC_RESTRICTED, Boolean.TRUE.toString() );
                }
                else if( elRoleNameCount == 1 && elRoleNameList.get( 0 ).getText().equals( ANY_ROLE_WILDCARD ) && ! hasMetaComment )
                {
                    elAuthConstraint.setMetaCommentText( MC_RESTRICTED, Boolean.TRUE.toString() );
                    elRoleNameList.get( 0 ).remove();
                }
                else
                {
                    elAuthConstraint.setMetaCommentText( MC_RESTRICTED, null );
                }
            }
            
            return elAuthConstraint;
        }
        
        @Override
        public void remove()
        {
            final XmlElement elSecurityConstraint = ( (XmlResource) property().element().resource() ).getXmlElement();
            elSecurityConstraint.removeChildNode( EL_AUTH_CONSTRAINT );
        }
    }
    
    public static final class RestrictedAccessRolesBinding extends StandardXmlListBindingImpl
    {
        @Override
        protected void initBindingMetadata()
        {
            final XmlNamespaceResolver xmlNamespaceResolver = ( (XmlResource) property().element().resource() ).getXmlNamespaceResolver();
            
            this.modelElementTypes = new ElementType[] { SecurityConstraint.RestrictedAccess.RoleRef.TYPE };
            this.xmlElementNames = new QName[] { createQualifiedName( EL_ROLE_NAME, xmlNamespaceResolver ) };
        }

        @Override
        protected Object insertUnderlyingObject( final ElementType type, final int position )
        {
            // After adding a "role-name" element, remove the meta comment as restricted access
            // is implied when at least one "role-name" element is present.

            final XmlElement element = (XmlElement) super.insertUnderlyingObject( type, position );
            element.getParent().setMetaCommentText( MC_RESTRICTED, null );
            
            return element;
        }

        @Override
        public void remove( final Resource resource )
        {
            // Prior to removing last "role-name" element, set the meta comment to avoid
            // transition from restricted access to closed access.
            
            final XmlResource xmlResource = (XmlResource) resource;
            final XmlElement xmlElement = xmlResource.getXmlElement();
            final XmlElement xmlElementParent = xmlElement.getParent();
            
            if( xmlElementParent.getChildElements( EL_ROLE_NAME ).size() == 1 )
            {
                xmlElementParent.setMetaCommentText( MC_RESTRICTED, Boolean.TRUE.toString() );
            }
            
            super.remove( resource );
        }
    }
    
}
