/******************************************************************************
 * Copyright (c) 2013 Oracle
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Konstantin Komissarchik - initial implementation and ongoing maintenance
 ******************************************************************************/

package org.eclipse.jst.config.web.internal;

import static org.eclipse.sapphire.modeling.util.MiscUtil.equal;
import static org.eclipse.sapphire.modeling.xml.XmlUtil.changeNamespace;
import static org.eclipse.sapphire.modeling.xml.XmlUtil.convertFromNamespaceForm;
import static org.eclipse.sapphire.modeling.xml.XmlUtil.convertToNamespaceForm;

import java.util.List;

import org.eclipse.sapphire.modeling.xml.XmlElement;
import org.eclipse.sapphire.modeling.xml.XmlValueBindingImpl;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * @author <a href="mailto:konstantin.komissarchik@oracle.com">Konstantin Komissarchik</a>
 */

public final class ServletPlatformVersionBinding extends XmlValueBindingImpl
{
    private static final String EL_WEB_APP = "web-app";
    private static final String EL_JSP_CONFIG = "jsp-config";
    private static final String EL_TAGLIB = "taglib";
    private static final String EL_TAGLIB_LOCATION = "taglib-location";
    private static final String EL_TAGLIB_URI = "taglib-uri";
    private static final String ATTR_VERSION = "version";
    private static final String META_COMMENT_NAME = "servlet-platform-version";
    
    private static final String VERSION_2_2 = "2.2";
    private static final String VERSION_2_2_PUBLIC_ID = "-//Sun Microsystems, Inc.//DTD Web Application 2.2//EN";
    private static final String VERSION_2_2_SYSTEM_ID = "http://java.sun.com/j2ee/dtds/web-app_2_2.dtd";
    
    private static final String VERSION_2_3 = "2.3";
    private static final String VERSION_2_3_PUBLIC_ID = "-//Sun Microsystems, Inc.//DTD Web Application 2.3//EN";
    private static final String VERSION_2_3_SYSTEM_ID = "http://java.sun.com/dtd/web-app_2_3.dtd";
    
    private static final String VERSION_2_4 = "2.4";
    private static final String VERSION_2_4_NAMESPACE = "http://java.sun.com/xml/ns/j2ee";
    private static final String VERSION_2_4_SCHEMA = "http://java.sun.com/xml/ns/j2ee/web-app_2_4.xsd";
    
    private static final String VERSION_2_5 = "2.5";
    private static final String VERSION_2_5_NAMESPACE = "http://java.sun.com/xml/ns/javaee";
    private static final String VERSION_2_5_SCHEMA = "http://java.sun.com/xml/ns/javaee/web-app_2_5.xsd";
    
    private static final String VERSION_3_0 = "3.0";
    private static final String VERSION_3_0_NAMESPACE = "http://java.sun.com/xml/ns/javaee";
    private static final String VERSION_3_0_SCHEMA = "http://java.sun.com/xml/ns/javaee/web-app_3_0.xsd";
    
    @Override
    public String read()
    {
        final XmlElement root = xml();
        
        if( root != null )
        {
            final Document document = root.getDomNode().getOwnerDocument();
            final DocumentType doctype = document.getDoctype();
            
            if( doctype == null )
            {
                final String namespace = root.getQualifiedName().getNamespaceURI();
                
                if( equal( namespace, VERSION_2_4_NAMESPACE ) )
                {
                    if( equal( root.getSchemaLocation(), VERSION_2_4_SCHEMA ) )
                    {
                        return VERSION_2_4;
                    }
                }
                else if( equal( namespace, VERSION_2_5_NAMESPACE ) )
                {
                    final String schema = root.getSchemaLocation();
                    
                    if( equal( schema, VERSION_2_5_SCHEMA ) )
                    {
                        return VERSION_2_5;
                    }
                    else if( equal( schema, VERSION_3_0_SCHEMA ) )
                    {
                        return VERSION_3_0;
                    }
                }
            }
            else
            {
                final String publicId = doctype.getPublicId();
                final String systemId = doctype.getSystemId();
                
                if( equal( publicId, VERSION_2_2_PUBLIC_ID ) && equal( systemId, VERSION_2_2_SYSTEM_ID ) )
                {
                    return VERSION_2_2;
                }
                else if( equal( publicId, VERSION_2_3_PUBLIC_ID ) && equal( systemId, VERSION_2_3_SYSTEM_ID ) )
                {
                    return VERSION_2_3;
                }
            }
    
            return root.getMetaCommentText( META_COMMENT_NAME );
        }
        
        return null;
    }

    @Override
    public void write( final String value )
    {
        String newPublicId = null;
        String newSystemId = null;
        String newNamespace = null;
        String newSchemaLocation = null;
        String newMetaComment = null;
        
        boolean nestTaglib = false;
        boolean unnestTaglib = false;
        
        if( value != null )
        {
            if( value.equals( VERSION_2_2 ) )
            {
                newPublicId = VERSION_2_2_PUBLIC_ID;
                newSystemId = VERSION_2_2_SYSTEM_ID;
                unnestTaglib = true;
            }
            else if( value.equals( VERSION_2_3 ) )
            {
                newPublicId = VERSION_2_3_PUBLIC_ID;
                newSystemId = VERSION_2_3_SYSTEM_ID;
                unnestTaglib = true;
            }
            else if( value.equals( VERSION_2_4 ) )
            {
                newNamespace = VERSION_2_4_NAMESPACE;
                newSchemaLocation = VERSION_2_4_SCHEMA;
                nestTaglib = true;
            }
            else if( value.equals( VERSION_2_5 ) )
            {
                newNamespace = VERSION_2_5_NAMESPACE;
                newSchemaLocation = VERSION_2_5_SCHEMA;
                nestTaglib = true;
            }
            else if( value.equals( VERSION_3_0 ) )
            {
                newNamespace = VERSION_3_0_NAMESPACE;
                newSchemaLocation = VERSION_3_0_SCHEMA;
                nestTaglib = true;
            }
            else
            {
                newMetaComment = value;
            }
        }
        
        final Document document = xml( true ).getDomNode().getOwnerDocument();
        final DocumentType oldDocType = document.getDoctype();
        
        Node nodeAfterDocType = null;

        if( oldDocType != null )
        {
            nodeAfterDocType = oldDocType.getNextSibling();
            
            while( nodeAfterDocType instanceof Text && nodeAfterDocType.getNodeValue().trim().length() == 0 )
            {
                document.removeChild( nodeAfterDocType );
                nodeAfterDocType = oldDocType.getNextSibling();
            }
            
            document.removeChild( oldDocType );
        }
        
        if( nodeAfterDocType == null )
        {
            nodeAfterDocType = xml( true ).getDomNode();
        }
        
        if( newPublicId != null )
        {
            final DocumentType newDocType = document.getImplementation().createDocumentType( EL_WEB_APP, newPublicId, newSystemId );
            document.insertBefore( newDocType, nodeAfterDocType );
            
            final Text newLineTextNode = document.createTextNode( "\n" );
            document.insertBefore( newLineTextNode, nodeAfterDocType );
        }
        
        final String oldNamespace = xml( true ).getNamespace();
        
        if( oldNamespace == null )
        {
            if( newNamespace != null )
            {
                convertToNamespaceForm( document, newNamespace, newSchemaLocation );
            }
        }
        else
        {
            if( newNamespace == null )
            {
                convertFromNamespaceForm( document );
            }
            else
            {
                changeNamespace( document, oldNamespace, newNamespace, newSchemaLocation );
            }
        }
        
        final XmlElement root = xml( true );
        
        root.setMetaCommentText( META_COMMENT_NAME, newMetaComment );
        
        String versionAttributeValue = null;
        
        if( value != null && ( value.equals( VERSION_2_4 ) || value.equals( VERSION_2_5 ) || value.equals( VERSION_3_0 ) ) )
        {
            versionAttributeValue = value;
        }
        
        root.setAttributeText( ATTR_VERSION, versionAttributeValue, true );
        
        if( nestTaglib )
        {
            final List<XmlElement> taglibs = root.getChildElements( EL_TAGLIB );
            
            if( ! taglibs.isEmpty() )
            {
                final XmlElement jspconfig = root.getChildElement( EL_JSP_CONFIG, true );
                
                for( XmlElement taglibInOldLocation : taglibs )
                {
                    final XmlElement taglibInNewLocation = jspconfig.addChildElement( EL_TAGLIB );
                    taglibInNewLocation.setChildNodeText( EL_TAGLIB_LOCATION, taglibInOldLocation.getChildNodeText( EL_TAGLIB_LOCATION ), true );
                    taglibInNewLocation.setChildNodeText( EL_TAGLIB_URI, taglibInOldLocation.getChildNodeText( EL_TAGLIB_URI ), true );
                    taglibInOldLocation.remove();
                }
            }
        }

        if( unnestTaglib )
        {
            final XmlElement jspconfig = root.getChildElement( EL_JSP_CONFIG, false );
            
            if( jspconfig != null )
            {
                final List<XmlElement> taglibs = jspconfig.getChildElements( EL_TAGLIB );
                
                if( ! taglibs.isEmpty() )
                {
                    for( XmlElement taglibInOldLocation : taglibs )
                    {
                        final XmlElement taglibInNewLocation = root.addChildElement( EL_TAGLIB );
                        taglibInNewLocation.setChildNodeText( EL_TAGLIB_LOCATION, taglibInOldLocation.getChildNodeText( EL_TAGLIB_LOCATION ), true );
                        taglibInNewLocation.setChildNodeText( EL_TAGLIB_URI, taglibInOldLocation.getChildNodeText( EL_TAGLIB_URI ), true );
                        taglibInOldLocation.remove();
                    }
                }
                
                if( jspconfig.getChildElements().isEmpty() )
                {
                    jspconfig.remove();
                }
            }
        }
    }
    
}
